.root
    vertical-align: middle
    fill: currentColor
    height: auto
    position: relative
    bottom: 1px

.red
    color: var(--color-red-text)

.green
    color: var(--color-green-text)

.emoji
    padding-right: 0.2em
