.root
    width: 100%
    padding: 3.5rem 10% 1.5rem
    font-size: var(--font-size-xs)
    font-family: var(--font-primary)
    color: var(--color-theme-dark-secondary)
    font-weight: bold
    text-align: center

.meta
    list-style: none
    text-transform: uppercase

    & > li
        display: inline-block
        margin-right: 2.5rem
        margin-bottom: 2rem
