/* Fonts */

@font-face
    font-family: "Calibre"
    font-style: normal
    font-weight: 400
    font-display: fallback
    src: url("../fonts/calibre-regular.eot")
    src: url("../fonts/calibre-regular.eot?#iefix") format("embedded-opentype"), url("../fonts/calibre-regular.woff2") format("woff2"), url("../fonts/calibre-regular.woff") format("woff")

@font-face
    font-family: "Calibre"
    font-style: italic
    font-weight: 400
    font-display: fallback
    src: url("../fonts/calibre-regular-italic.eot")
    src: url("../fonts/calibre-regular-italic.eot?#iefix") format("embedded-opentype"), url("../fonts/calibre-regular-italic.woff2") format("woff2"), url("../fonts/calibre-regular-italic.woff") format("woff")

@font-face
    font-family: "Calibre"
    font-style: normal
    font-weight: 700
    font-display: fallback
    src: url("../fonts/calibre-semibold.eot")
    src: url("../fonts/calibre-semibold.eot?#iefix") format("embedded-opentype"), url("../fonts/calibre-semibold.woff2") format("woff2"), url("../fonts/calibre-semibold.woff") format("woff")

@font-face
    font-family: "Calibre"
    font-style: italic
    font-weight: 700
    font-display: fallback
    src: url("../fonts/calibre-semibold-italic.eot")
    src: url("../fonts/calibre-semibold-italic.eot?#iefix") format("embedded-opentype"), url("../fonts/calibre-semibold-italic.woff2") format("woff2"), url("../fonts/calibre-semibold-italic.woff") format("woff")

@font-face
    font-family: "IBM Plex Mono"
    font-style: normal
    font-weight: 400
    font-display: swap
    src: local("IBM Plex Mono Medium"), local("IBMPlexMono-Medium"), url("../fonts/IBMPlexMono-Medium.woff2") format("woff2"), url("../fonts/IBMPlexMono-Medium.woff") format("woff")

@font-face
    font-family: "IBM Plex Mono"
    font-style: italic
    font-weight: 400
    font-display: swap
    src: local("IBM Plex Mono Medium Italic"), local("IBMPlexMono-MediumItalic"), url("../fonts/IBMPlexMono-MediumItalic.woff2") format("woff2"), url("../fonts/IBMPlexMono-MediumItalic.woff") format("woff")
