.root
    margin-bottom: 4rem

.header
    margin-bottom: 2rem

.input
    display: none

.chip
    cursor: pointer
    border-radius: 2em
    padding: 0.5rem 1rem
    margin-bottom: 1rem
    display: inline-block
    background: var(--color-subtle)
    color: var(--color-dark)
    font-weight: bold
    font-size: var(--font-size-sm)
    line-height: 1

    &:not(:last-child)
        margin-right: 1rem

.chip-active
    background: var(--color-theme-dark-secondary)
    color: var(--color-theme-contrast)
